CraftList2 = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "AceHook-2.1")
local L = AceLibrary("AceLocale-2.2"):new("CraftList2")
local G = AceLibrary("Gratuity-2.0")
local BTS= AceLibrary("Babble-Tradeskill-2.2")
local waterfall = AceLibrary("Waterfall-1.0")

local prefix_ench = "!enchant"
local prefix_gem = "!gem"
local prefix_other = "!craft"
local prefix_answers = "cL2>"

local LOCKED=0

function CraftList2:OnInitialize()
	CraftList2:RegisterDB("CraftList2DB")
	CraftList2:RegisterDefaults("realm", {
		data = {},
		ignored_crafts={},
		maxresults=10,
		showrequests=true,
		showanswers=true,
		locktime=10,
		}
	)  
	
	self.cl2opt = { 
		type="group",
		name="CraftList2",
		args = {
			inc= {
				type="toggle",
				name="Show incoming requests",
				desc="Show incoming requests",
				get=function() return self.db.realm.showrequests end,
				set=function(v) self.db.realm.showrequests = v end,
			},
			out= {
				type="toggle",
				name="Show your replies",
				desc="Show your replies",
				get=function() return self.db.realm.showanswers end,
				set=function(v) self.db.realm.showanswers = v end,
			},
			tlock= {
				type="range",
				name="Lock time between 2 requests (antispam)",
				desc="Lock time between 2 requests (antispam)",
				get=function() return self.db.realm.locktime end,
				set=function(v) self.db.realm.locktime = v end,
				min=1,
				max=30,
				step=1,
			},
			maxresults= {
				type="range",
				name="Maximum results",
				desc="Maximum of results sent to someone (0=no limit)",
				get=function() return self.db.realm.maxresults end,
				set=function(v) self.db.realm.maxresults = v end,
				min=0,
				max=100,
				step=1,
			},
			active= {
				type="group",
				name="Active Tradeskills",
				desc="Active TradeSkills",
				args = {},
			}				
		}
	}
	
	local k=1
	for character, tradeskill in pairs(self.db.realm.data) do
		for ts,tsinfo in pairs(tradeskill) do
			self.cl2opt.args.active.args[k] = 
			{
				type="toggle",
				name=character.." - "..ts,
				desc=character.." - "..ts,
				get=function() 
					if not self.db.realm.ignored_crafts[character] then self.db.realm.ignored_crafts[character] = {} end
					return not self.db.realm.ignored_crafts[character][ts] 
					end,
				set=function(v) 
					if not self.db.realm.ignored_crafts[character] then self.db.realm.ignored_crafts[character] = {} end
					self.db.realm.ignored_crafts[character][ts] = not self.db.realm.ignored_crafts[character][ts] 
					end,
			}
			k=k+1
		end
	end
	k=0	
	
	--self:RegisterChatCommand({"/clist", "/craftlist", "/clist2", "/craftlist2"}, self.cl2opt)
	
	--waterfall:FeedAceOptionsTable(self.cl2opt,'active', 1)
	waterfall:Register('CraftList2', 
		'aceOptions',self.cl2opt,
		'title','CraftList2')
	self:RegisterChatCommand('/clist', '/craftlist', {
		type = 'execute',
		func = function()
			waterfall:Open('CraftList2')
		end
	})
end

function CraftList2:OnEnable()
	self:Hook("ChatFrame_MessageEventHandler", "WhisperHandler", true)
	self:RegisterEvent("CHAT_MSG_WHISPER")
	self:RegisterEvent("CRAFT_SHOW")
	self:RegisterEvent("TRADE_SKILL_SHOW")
end

function CraftList2:OnDisable()
	self:UnhookAll()
	self:UnregisterEvent("CHAT_MSG_WHISPER")
	self:UnregisterEvent("CRAFT_SHOW")
	self:UnregisterEvent("TRADE_SKILL_UPDATE")
end

function CraftList2:CRAFT_SHOW()
	local craftname= CraftFrameTitleText:GetText()
	self:ScanCraft(craftname)
end

function CraftList2:TRADE_SKILL_SHOW()
	local tsname= TradeSkillFrameTitleText:GetText()
	self:ScanTS(tsname)
end

function CraftList2:ScanCraft(craftname)
	local craftdata = {}
	for i=1,GetNumCrafts() do
		local skillname, craftSubSpellName, skilltype = GetCraftInfo(i)
		if skilltype~="header" and skillname then
			local craftinfo = {}
			local link = GetCraftItemLink(i)
			local desc = GetCraftDescription(i)
			if desc == nil then desc = "" end
		--[[	local reg = {}
			for j=1,GetCraftNumReagents(i) do
				table.insert(reg,GetCraftReagentItemLink(i,j))
			end
			craftinfo["reg"] = reg ]]
			craftinfo["link"] = link
			craftinfo["desc"] = desc
			craftdata[skillname]=craftinfo
		end	
	end
	if not self.db.realm.data[UnitName("player")] then self.db.realm.data[UnitName("player")] = {} end
	self.db.realm.data[UnitName("player")][craftname] = craftdata
end

function CraftList2:ScanTS(tsname)
	local tsdata = {}
	for i=1,GetNumTradeSkills() do
		local skillname, skilltype, numAvailable, isExpanded = GetTradeSkillInfo(i)
		if skilltype ~= "header" then
			local tsinfo = {}
			local itemlink = GetTradeSkillItemLink(i)
			local recipelink = GetTradeSkillRecipeLink(i)
			local reg = {}
			for j=1,GetTradeSkillNumReagents(i) do
				table.insert(reg,GetTradeSkillReagentItemLink(i,j))
			end
			tsinfo["itemlink"] = itemlink
			tsinfo["recipelink"] = recipelink
			tsinfo["reg"] = reg
			tsdata[skillname]=tsinfo
		end	
	end
	if not self.db.realm.data[UnitName("player")] then self.db.realm.data[UnitName("player")] = {} end
	self.db.realm.data[UnitName("player")][tsname] = tsdata
end


function CraftList2:CHAT_MSG_WHISPER()
	local argum = self:GetArgs(arg1," ")
	local sender = arg2
	local firstword = argum[1]
	table.remove(argum,1)
	argum=self:RecomposeLink(argum)
	if firstword == prefix_ench then self:HandleEnchant(argum,sender)
	elseif firstword == prefix_gem then self:HandleGem(argum,sender)
	elseif firstword == prefix_other then self:HandleCraft(argum,sender)
	else return
	end
end

function CraftList2:WhisperHandler(event)
	if arg1 and arg1~="" then
		if (string.sub(arg1,1,string.len(prefix_ench))==prefix_ench and not self.db.realm.showrequests and event=="CHAT_MSG_WHISPER") then return end
		if (string.sub(arg1,1,string.len(prefix_gem))==prefix_gem and not self.db.realm.showrequests and event=="CHAT_MSG_WHISPER") then return end
		if (string.sub(arg1,1,string.len(prefix_other))==prefix_other and not self.db.realm.showrequests and event=="CHAT_MSG_WHISPER") then return end
		if (string.sub(arg1,1,string.len(prefix_answers))==prefix_answers and not self.db.realm.showanswers and event=="CHAT_MSG_WHISPER_INFORM") then return end
	end
	return self.hooks["ChatFrame_MessageEventHandler"](event)
end

function CraftList2:GetArgs(text, separator)
	local args = {}
	local i = 0
	for value in string.gmatch(text, "[^"..separator.."]+") do
		i = i + 1
		args[i] = value
	end	
	return args
end

function CraftList2:RecomposeLink(argument)
	local message=argument
	for k,v in pairs(message) do
		if strfind(v,"|cff") then
				if strfind(v,"|h|r") then return message
				else 
					message[k]=v.." "..message[k+1]
					table.remove(message,k+1)
					return self:RecomposeLink(message)
				end
		end
	end
	return message
end

function CraftList2:Respond(message,sender)
	SendChatMessage(prefix_answers.." "..message, "WHISPER", nil, sender)
end

function CraftList2:Lock()
	LOCKED=1
	self:ScheduleEvent(function() LOCKED=0 return end, self.db.realm.locktime)
end


function CraftList2:HandleEnchant(argum,sender)
	if #argum == 0 then self:Respond("Usage: !enchant <any combination of keywords and itemlinks>",sender) return end

	-- replace an itemlink with the piece of armor 
	for k,v in pairs(argum) do
		if strfind(argum[k],"|cff") and strfind(argum[k],"|h|r") then
			local sName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount, iel = GetItemInfo(argum[k])
			if sType == L["Armor"] then 
				local b=0
				if iel=="INVTYPE_CHEST" then b=L["Chest"]
				elseif iel=="INVTYPE_ROBE" then b=L["Chest"]
				elseif iel=="INVTYPE_FEET" then b=L["Boots"]
				elseif iel=="INVTYPE_WRIST" then b=L["Bracer"]
				elseif iel=="INVTYPE_HAND" then b=L["Gloves"]
				elseif iel=="INVTYPE_CLOAK" then b=L["Cloak"]
				elseif iel=="INVTYPE_SHIELD" then b=L["Shield"]
				elseif iel=="INVTYPE_FINGER" then b=L["Ring"]
				else self:Respond("Error! Cannot enchant that.",sender) return end
				argum[k]=b
			elseif sType==L["Weapon"] then 
				local b=0
				if iel=="INVTYPE_2HWEAPON" then b=L["Weapon"]
				elseif iel=="INVTYPE_WEAPONMAINHAND" then b=L["Enchant Weapon"]
				elseif iel=="INVTYPE_WEAPONOFFHAND" then b=L["Enchant Weapon"]
				elseif iel=="INVTYPE_WEAPON" then b=L["Enchant Weapon"]
				else self:Respond("Error! Cannot enchant that.",sender) return end
				argum[k]=b
			end
		end
	end
	-- done replacing
	
	-- building the results
	local searchresult={}
	if self.db.realm.data=={} then self:Respond("Database is currently empty.",sender) return end
	
	for character, tradeskill in pairs(self.db.realm.data) do
		if not self.db.realm.ignored_crafts[character] then self.db.realm.ignored_crafts[character] = {} end
		if tradeskill[BTS["Enchanting"]] and not self.db.realm.ignored_crafts[character][BTS["Enchanting"]] then
			for name,info in pairs(tradeskill[BTS["Enchanting"]]) do
				local match=0
				local enchantname=string.lower(name)
				local enchantdesc=string.lower(info["desc"])
				for j=1,#argum do
					if strfind(enchantname, string.lower(argum[j])) then match=match+1 
					elseif strfind(enchantdesc, string.lower(argum[j])) then match=match+1
					end
				end 
				if match==#argum then table.insert(searchresult,info["link"]) end	
			end	
		end
	end
	self:SendResults(searchresult,sender)
end

function CraftList2:HandleGem(argum,sender)
	if #argum == 0 then self:Respond("Usage: !gem color/stat/reagent/text",sender) return end
	
	local searchresult={}
	if self.db.realm.data=={} then self:Respond("Database is currently empty.",sender) return end
	
	
	for k,v in pairs(argum) do
		if strfind(argum[k],"|cff") and strfind(argum[k],"|h|r") then
			local sName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount, iel = GetItemInfo(argum[k])
			argum[k] = sName
		end
	end


	for character, tradeskill in pairs(self.db.realm.data) do
		if not self.db.realm.ignored_crafts[character] then self.db.realm.ignored_crafts[character] = {} end
		if tradeskill[BTS["Jewelcrafting"]] and not self.db.realm.ignored_crafts[character][BTS["Jewelcrafting"]] then
			for name,info in pairs(tradeskill[BTS["Jewelcrafting"]]) do
				local sName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount, iel = GetItemInfo(info["itemlink"])
				if sType== "Gem" then
					local match=0
					local gemname=string.lower(name)
					G:SetHyperlink(info["itemlink"])
					local gemdesc=string.lower(G:GetLine(2))
					local gemcolor=string.lower(G:GetLine(G:NumLines()) )
					local reg = info["reg"]
					for j=1,#argum do
						if strfind(gemname, string.lower(argum[j])) then match=match+1 
						elseif strfind(gemdesc, string.lower(argum[j])) then match=match+1
						elseif strfind(gemcolor,string.lower(argum[j])) then match = match+1
						else 
							for k,v in pairs(reg) do
								if strfind(v,argum[j]) then match = match+1 end
							end
						end
					end 
					if match==#argum then table.insert(searchresult,info["itemlink"]) end	
				end
			end	
		end
	end

	self:SendResults(searchresult,sender)
end

function CraftList2:HandleCraft(argum,sender)
	if #argum == 0 then self:Respond("Usage: !craft reagent/text",sender) return end
	
	local searchresult={}
	if self.db.realm.data=={} then self:Respond("Database is currently empty.",sender) return end
	
	
	for k,v in pairs(argum) do
		if strfind(argum[k],"|cff") and strfind(argum[k],"|h|r") then
			local sName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount, iel = GetItemInfo(argum[k])
			argum[k] = sName
		end
	end


	for character, tradeskill in pairs(self.db.realm.data) do
		for ts,tsinfo in pairs(tradeskill) do
			if not self.db.realm.ignored_crafts[character] then self.db.realm.ignored_crafts[character] = {} end
			if not self.db.realm.ignored_crafts[character][ts] then
				for name,info in pairs(tsinfo) do
					local match=0
					local craftname=string.lower(name)
					local reg = info["reg"]
					for j=1,#argum do
						if strfind(craftname, string.lower(argum[j])) then match=match+1 
						else 
							if reg then
								for k,v in pairs(reg) do
									if strfind(v,argum[j]) then match = match+1 end
								end
							end
						end
					end 
					if match==#argum then 
						if info["recipelink"] then table.insert(searchresult,info["recipelink"]) 
						else table.insert(searchresult,info["link"]) 
						end
					end	
				end
			end
		end
	end

	self:SendResults(searchresult,sender)
end

function CraftList2:SendResults(searchresult,sender)
	if LOCKED == 1 then self:Respond( ("Wait ... %d seconds lock (antispam)"):format(self.db.realm.locktime) ,sender)
	else 
		self:Lock()
		if #searchresult == 0 then self:Respond("Nothing found.",sender)
		else 
			local max = self.db.realm.maxresults
			if #searchresult < max then max = #searchresult end
			if max == 0 then max = #searchresult end
			for i=1,max do
				self:Respond(searchresult[i],sender)
			end
		end
	end
end
